package core

import (
	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/cpu"
	"git.akyoto.dev/cli/q/src/build/errors"
	"git.akyoto.dev/cli/q/src/build/token"
)

// opRegisterNumber performs an operation on a register and a number.
func (f *Function) opRegisterNumber(operation token.Token, register cpu.Register, number int) error {
	switch operation.Text() {
	case "+", "+=":
		f.assembler.RegisterNumber(asm.ADD, register, number)

	case "-", "-=":
		f.assembler.RegisterNumber(asm.SUB, register, number)

	case "*", "*=":
		f.assembler.RegisterNumber(asm.MUL, register, number)

	case "/", "/=":
		f.assembler.RegisterNumber(asm.DIV, register, number)

	case "=":
		f.assembler.RegisterNumber(asm.MOVE, register, number)

	default:
		return errors.New(&errors.InvalidOperator{Operator: operation.Text()}, f.File, operation.Position)
	}

	return nil
}

// opRegisterRegister performs an operation on two registers.
func (f *Function) opRegisterRegister(operation token.Token, destination cpu.Register, source cpu.Register) error {
	switch operation.Text() {
	case "+", "+=":
		f.assembler.RegisterRegister(asm.ADD, destination, source)

	case "-", "-=":
		f.assembler.RegisterRegister(asm.SUB, destination, source)

	case "*", "*=":
		f.assembler.RegisterRegister(asm.MUL, destination, source)

	case "/", "/=":
		f.assembler.RegisterRegister(asm.DIV, destination, source)

	case "=":
		if destination != source {
			f.assembler.RegisterRegister(asm.MOVE, destination, source)
		}

	default:
		return errors.New(&errors.InvalidOperator{Operator: operation.Text()}, f.File, operation.Position)
	}

	return nil
}
